/*
 *	fscl.c
 *
 *	Check if we need to run fsck on a file system
 *
 *	Syntax:	fscl [-clean] device
 *
 *	Returns: 0 if file system is ok
 *	         1 if file system is not ok
 *	         2 if open, read or write errors
 *	         3 if usage errors
 *
 *	Note: The -clean option forces a disk to be mountable without
 *	      running fsck. It should be used with great care. One of
 *	      it's uses is to save files from a disk that have been
 *	      damaged so badly that fsck crashes (yes, this can happen.)
 *	      With this option it is possible to mount the disk anyway
 *	      and rescue as much as possible before mkfs-ing a new
 *	      filesystem on the disk.
 *	      Another use is to skip fsck after a system crash to speed
 *	      up the boot. This should only be used if you are sure that
 *	      the disk is ok (how can you be sure about that?) If the
 *	      disk was damaged, this may lead to more damages.
 *	                      YOU HAVE BEEN WARNED!
 *	      Without the -clean option, this program does the same as
 *	      the standard fscl in abcenix 5.18.
 *
 *	Author: hacker@isadora.ikp.liu.se (Goran Larsson)
 */
#include <sys/types.h>
#include <sys/sysfile.h>
#include <dnix/diskpar.h>
#include <fcntl.h>

int dev;
char *devname;
struct sysfptr buffer;
int force_clean = 0;

main ( argc, argv )
int argc;
char *argv[];

{
	/*
	** parse arguments
	*/

	switch ( argc ) {
	case 2:
		devname = argv[1];
		break;
	case 3:
		if ( strcmp ( argv[1], "-clean" ) != 0 ) {
			usage ();
		}
		force_clean++;
		devname = argv[2];
		if ( getuid () != 0 ) {
			printf ( "fscl: no permission\n" );
			exit ( 3 );
		}
		break;
	default:
		usage ();
	}

	/*
	** open device
	*/

	if ( ( dev = open ( devname, force_clean ? O_RDWR : O_RDONLY ) ) < 0 ) {
		printf ( "fscl: cannot open %s\n", devname );
		exit ( 2 );
	}

	/*
	** read file system pointer block
	*/

	if ( lseek ( dev, SYSFPTR, 0 ) < 0 ) {
		printf ( "fscl: error at seek\n" );
		exit ( 2 );
	}
	if ( read ( dev, &buffer, sizeof buffer ) != sizeof buffer ) {
		printf ( "fscl: error at read\n" );
		exit(2);
	}

	/*
	** if force clean operation, then set the flag to a clean
	** state if the volume is dirty, else just report the flags status
	*/

	if ( force_clean ) {
		if ( buffer.cleanfl == CLEANFL ) {
			printf (
			  "fscl: file system on device %s is ok\n",
			  devname
			);
		} else {
			if ( lseek ( dev, SYSFPTR, 0 ) < 0 ) {
				printf ( "fscl: error at seek\n" );
				exit ( 2 );
			}
			buffer.cleanfl = CLEANFL;
			if ( write ( dev, &buffer, sizeof buffer ) != sizeof buffer ) {
				printf ( "fscl: error at write\n" );
				exit(2);
			}
			printf (
			  "fscl: file system on device %s set to ok\n",
			  devname
			);
		}
		exit ( 0 );
	} else {
		printf (
		  "fscl: file system on device %s is %sok\n",
		  devname, ( buffer.cleanfl == CLEANFL ) ? "" : "not "
		);
		exit ( ( buffer.cleanfl == CLEANFL ) ? 0 : 1 );
	}
}

usage ()

{
	printf ( "usage: fscl [-clean] device\n" );
	exit ( 3 );
}
